package edu.berkeley.sbp.util;
import edu.berkeley.sbp.util.*;
import edu.berkeley.sbp.*;
import java.io.*;
import java.util.*;
import java.lang.reflect.*;
import java.lang.ref.*;

public class GraphViz {

    IdentityHashMap<ToGraphViz,Node> ihm = new IdentityHashMap<ToGraphViz,Node>();
    HashMap<Node,Group> groups = new HashMap<Node,Group>();

    public class Group {
        public Group() { }
        public void add(Node n) { groups.put(n, this); }
    }

    private static int master_idx=0;
    public class Node {
        private final int idx = master_idx++;
        public String label;
        public String comment;
        public boolean directed = false;
        public String color="black";
        public ArrayList<Node> edges = new ArrayList<Node>();
        public ArrayList<Object> labels = new ArrayList<Object>();
        public ArrayList<Node> inbound = new ArrayList<Node>();
        public void edge(ToGraphViz o, Object label) {
            Node n = o.toGraphViz(GraphViz.this);
            if (n==null) return;
            edges.add(n);
            labels.add(label);
            n.inbound.add(this);
        }
        public String getParentName() {
            if (inbound.size()==1 && inbound.get(0).simple())
                return inbound.get(0).getParentName();
            return name();
        }
        public String name() {
            if (inbound.size()==1 && inbound.get(0).simple())
                return inbound.get(0).getParentName()+":node_"+idx;
            return "node_"+idx;
        }
        public void edges(PrintWriter pw) {
            if (simple()) return;
            for(int i=0; i<edges.size(); i++) {
                Node n = edges.get(i);
                Object label = labels.get(i);
                pw.println("    "+name()+" -> " + n.name() + " [color="+color+" "
                           +(label==null?"":("label=\""+StringUtil.escapify(label.toString(), "\\\"\r\n")+"\""))+ "];\n");
            }
        }
        public int numEdges() { return edges.size(); }
        public boolean simple() {
            boolean simple = true;
            if (label!=null && !label.equals("")) simple = false;
            if (simple)
                for(Node n : edges)
                    //if (n.numEdges()>0) { simple = false; break; }
                    if (n.inbound.size() > 1) { simple = false; break; }
            return simple;
        }
        public void dump(PrintWriter pw) {
            if (inbound.size() > 0) {
                boolean good = false;
                for(Node n : inbound)
                    if (!n.simple())
                        { good = true; break; }
                if (!good) return;
            }
            pw.print("    "+name());
            pw.print(" [");
            if (directed) pw.print("ordering=out");
            if (simple()) {
                pw.print(" shape=record ");
                pw.print(" label=\"");
                boolean complex = false;
                for(Node n : edges)
                    if (n.edges.size()>0)
                        complex = true;
                if (!complex) pw.print("{");
                boolean first = true;
                for(Node n : edges) {
                    if (!first) pw.print("|");
                    first = false;
                    pw.print("<node_"+n.idx+">");
                    pw.print(StringUtil.escapify(n.label,"\\\"\r\n"));
                }
                if (!complex) pw.print("}");
                pw.print("\" ");
            } else {
                pw.print(" label=\"");
                pw.print(StringUtil.escapify(label,"\\\"\r\n"));
                pw.print("\" ");
            }
            pw.print("color="+color);
            if (comment!=null) pw.print(" comment=\""+StringUtil.escapify(comment,"\\\"\r\n")+"\" ");
            pw.print("];\n");
        }
    }

    public boolean hasNode(ToGraphViz o) {
        return ihm.get(o)!=null;
    }

    public Node createNode(ToGraphViz o) {
        Node n = ihm.get(o);
        if (n!=null) return n;
        n = new Node();
        ihm.put(o, n);
        return n;
    }

    public static interface ToGraphViz {
        public Node    toGraphViz(GraphViz gv);
        public boolean isTransparent();
        public boolean isHidden();
    }

    public void show() throws IOException {
        Runtime.getRuntime().exec(new String[] { "dot", "-Tsvg" });
    }

    public void dump(OutputStream os) { dump(new PrintWriter(new OutputStreamWriter(os))); }
    public void dump(PrintWriter pw) {
        IdentityHashMap<Node,Node> done = new IdentityHashMap<Node,Node>();
        pw.println("digraph G { rankdir=LR; ordering=out; \n");
        for(Group g : groups.values()) {
            pw.println("  { rank=same;\n");
            for(Node n : groups.keySet())
                if (groups.get(n)==g) {
                    done.put(n,n);
                    n.dump(pw);
                }
            pw.println("  }\n");
        }
        for(Node n : ihm.values()) {
            if (done.get(n)!=null) continue;
            n.dump(pw);
        }
        for(Node n : ihm.values()) n.edges(pw);
        pw.println("}\n");
        pw.flush();
    }

}
